"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const constants_1 = require("next-server/constants");
const normalize_page_path_1 = require("./normalize-page-path");
function pageNotFoundError(page) {
    const err = new Error(`Cannot find module for page: ${page}`);
    err.code = 'ENOENT';
    return err;
}
exports.pageNotFoundError = pageNotFoundError;
function getPagePath(page, distDir) {
    const serverBuildPath = path_1.join(distDir, constants_1.SERVER_DIRECTORY);
    const pagesManifest = require(path_1.join(serverBuildPath, constants_1.PAGES_MANIFEST));
    try {
        page = normalize_page_path_1.normalizePagePath(page);
        page = page === '/' ? '/index' : page;
    }
    catch (err) {
        // tslint:disable-next-line
        console.error(err);
        throw pageNotFoundError(page);
    }
    if (!pagesManifest[page]) {
        throw pageNotFoundError(page);
    }
    return path_1.join(serverBuildPath, pagesManifest[page]);
}
exports.getPagePath = getPagePath;
function requirePage(page, distDir) {
    const pagePath = getPagePath(page, distDir);
    return require(pagePath);
}
exports.requirePage = requirePage;
